<?php
function evalTask1($qrel, $run, $checkdata, &$result) {
  $QREL = array();
  $DATA = array();
  $CONFUSION = array();

  if(!($fd = fopen($qrel, 'r'))) {
    echo 'Cannot open qrel file.'."\n";
    return false;
  }
  $qrel_size = 0;
  while(!feof($fd)) {
    $line = trim(fgets($fd), "\n\r");
    if(preg_match('/^(.+?)\t(.+?)$/', $line, $fields)) {
      $qrel_size++;
      $id = $fields[1];
      $value = $fields[2];
      $QREL[$id] = $value;
    } elseif(!empty($line)) {
      echo 'Wrong qrel file format: "'.$line.'".'."\n";
      return false;
    }
  }
  fclose($fd);
  if(!($fd = fopen($run, 'r'))) {
    echo 'Cannot open run file.'."\n";
    return false;
  }
  while(!feof($fd)) {
    $line = trim(fgets($fd), "\n\r");
    if(preg_match('/^([^\t]+?)\t?$/', $line, $fields)) {
      $id = $fields[1];
      if(isset($QREL[$id]))
        ;
      elseif($checkdata) {
        echo 'Unknown instance id: "'.$id.'".'."\n";
        return false;
      }
    } elseif(preg_match('/^(.+?)\t(.+?)$/', $line, $fields)) {
      $id = $fields[1];
      $value = $fields[2];
      if(isset($QREL[$id])) 
        $DATA[$id] = $value;
      elseif($checkdata) {
        echo 'Unknown instance id: "'.$id.'".'."\n";
        return false;
      }
    } elseif(!empty($line)) {
      echo 'Wrong run file format: "'.$line.'".'."\n";
      return false;
    }
  }
  fclose($fd);

  $tp = 0;
  $fp = 0;
  $fn = 0;
  foreach($QREL as $id => $valueqrel) {
    if(isset($DATA[$id])) {
      $value = $DATA[$id];
      if($valueqrel==$value)
        $tp++;
      else
        $fp++;
    } else {
      $value = '?';
      $fn++;
    }
    if(!isset($CONFUSION[$value])) 
      $CONFUSION[$value] = array($valueqrel => 1);
    elseif(!isset($CONFUSION[$value][$valueqrel]))
      $CONFUSION[$value][$valueqrel] = 1;
    else
      $CONFUSION[$value][$valueqrel]++;
  }
  $a = $tp/$qrel_size;
  $p = $tp/($tp+$fp);
  $r = $tp/($tp+$fn);
  if($p+$r==0)
    $f1 = 0;
  else
    $f1 = 2*$p*$r/($p+$r);
  $result = array('a' => $a,
                  'p' => $p,
                  'r' => $r,
                  'f1' => $f1,
                  'info' => '');
  $result['info'] = sizeof($QREL).' instances in qrel, '.$qrel_size.' annotations in qrel, '.($tp+$fp).' predictions.'."\n".
                    "\n".
                    'Accuracy: '.number_format($a, 3)."\n".
                    'Microaveraged Precision: '.number_format($p, 3)."\n".
                    'Microaveraged Recall: '.number_format($r, 3)."\n".
                    'Microaveraged F1: '.number_format($f1, 3)."\n".
                    "\n".
                    'Confusion matrix'."\n".
                    'Pred'."\t".'Actual'."\t".'#'."\n";
  ksort($CONFUSION);
  foreach($CONFUSION as $k1 => $v1) {
    foreach($v1 as $k2 => $v2) 
      $result['info'] .= $k1."\t".$k2."\t".$v2."\n";
  }
  return $result;
} // evalTask1

if(isset($argv)) {
  if(sizeof($argv)-1!=2) {
    echo 'Usage: php eval-task1.php <qrel_file> <run_file>'."\n";
    exit(1);
  }
  if(evalTask1($argv[1], $argv[2], true, $result))
    print_r($result);
}
?>
